@@ -14,7 +14,7 @@ use Getopt::Std;
 use JSON;
 use Net::Shadowsocks::Client;
 
-my $version = "0.8.1";
+my $version = "0.9.0";
 
 sub main::HELP_MESSAGE()
 {
@@ -27,13 +27,17 @@ sub main::HELP_MESSAGE()
     print("\t-l LOCAL_PORT\t\tYour browser`s socks5 proxy port\n");
     print("\t-k PASSWORD\t\tYour top-secret password\n");
     print("\t-m METHOD\t\tYour favarite encryption method, pick one of the methods below:\n");
-    print("\taes-128-cbc aes-128-cfb aes-128-ctr aes-128-ofb\n");
-    print("\taes-192-cbc aes-192-cfb aes-192-ctr aes-192-ofb\n");
-    print("\taes-256-cbc aes-256-cfb aes-256-ctr aes-256-ofb\n");
-    print("\tcamellia-128-cbc camellia-128-cfb camellia-128-ctr camellia-128-ofb\n");
-    print("\tcamellia-192-cbc camellia-192-cfb camellia-192-ctr camellia-192-ofb\n");
-    print("\tcamellia-256-cbc camellia-256-cfb camellia-256-ctr camellia-256-ofb\n");
-    print("\tchacha20-ietf rc4-md5 rc4-sha rc6\n");
+    print("\taes-128-cfb aes-128-ctr aes-128-gcm aes-128-ofb\n");
+    print("\taes-192-cfb aes-192-ctr aes-192-gcm aes-192-ofb\n");
+    print("\taes-256-cfb aes-256-ctr aes-256-gcm aes-256-ofb\n");
+    print("\tcamellia-128-cfb camellia-128-ctr camellia-128-ofb\n");
+    print("\tcamellia-192-cfb camellia-192-ctr camellia-192-ofb\n");
+    print("\tcamellia-256-cfb camellia-256-ctr camellia-256-ofb\n");
+    print("\tchacha20-ietf chacha20-ietf-poly1305\n");
+    print("\trc4-md5 rc4-sha\n");
+    print("\trc6-128-cfb rc6-128-ctr rc6-128-ofb\n");
+    print("\trc6-192-cfb rc6-192-ctr rc6-192-ofb\n");
+    print("\trc6-256-cfb rc6-256-ctr rc6-256-ofb\n");
     printf("\t-c CONFIG\t\tFull path of your config.json file\n");
     print("\t-h, --help\t\tDisplay this help screen\n");
     print("\t-v, --version\t\tDisplay version information\n");
@@ -103,12 +107,10 @@ ssclient.pl
 
 =head1 VERSION
 
-Version 0.8.1
+Version 0.9.0
 
 =head1 SYNOPSIS
 
-Shadowsocks protocol client.
-
 Usage: ssclient.pl -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
     -l LOCAL_PORT -k PASSWORD -m METHOD [-c CONFIG]
 
@@ -120,13 +122,17 @@ Usage: ssclient.pl -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
 	-l LOCAL_PORT		Your browser`s socks5 proxy port
 	-k PASSWORD		Your top-secret password
 	-m METHOD		Your favarite encryption method, pick one of the methods below:
-	aes-128-cbc aes-128-cfb aes-128-ctr aes-128-ofb
-	aes-192-cbc aes-192-cfb aes-192-ctr aes-192-ofb
-	aes-256-cbc aes-256-cfb aes-256-ctr aes-256-ofb
-	camellia-128-cbc camellia-128-cfb camellia-128-ctr camellia-128-ofb
-	camellia-192-cbc camellia-192-cfb camellia-192-ctr camellia-192-ofb
-	camellia-256-cbc camellia-256-cfb camellia-256-ctr camellia-256-ofb
-	chacha20-ietf rc4-md5 rc4-sha rc6
+	aes-128-cfb aes-128-ctr aes-128-gcm aes-128-ofb
+	aes-192-cfb aes-192-ctr aes-192-gcm aes-192-ofb
+	aes-256-cfb aes-256-ctr aes-256-gcm aes-256-ofb
+	camellia-128-cfb camellia-128-ctr camellia-128-ofb
+	camellia-192-cfb camellia-192-ctr camellia-192-ofb
+	camellia-256-cfb camellia-256-ctr camellia-256-ofb
+	chacha20-ietf chacha20-ietf-poly1305
+	rc4-md5 rc4-sha
+	rc6-128-cfb rc6-128-ctr rc6-128-ofb
+	rc6-192-cfb rc6-192-ctr rc6-192-ofb
+	rc6-256-cfb rc6-256-ctr rc6-256-ofb
 	-c CONFIG		Full path of your config.json file
 	-h, --help		Display this help screen
 	-v, --version		Display version information
@@ -134,6 +140,13 @@ Usage: ssclient.pl -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
 
 =head1 DESCRIPTION
 
+Shadowsocks protocol client.
+
+=head1 SEE ALSO
+
+L<Shadowsocks Official website |https://shadowsocks.org/en/index.html>,L<Shadowsocks on Wikipedia |https://en.wikipedia.org/wiki/Shadowsocks>
+
+
 Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭)
 protocol client and server , Shadowsocks is a secure transport protocol based on
 SOCKS Protocol Version 5 (RFC 1928 ).
