@@ -14,7 +14,7 @@ use Getopt::Std;
 use JSON;
 use Net::Shadowsocks::Server;
 
-my $version = "0.8.1";
+my $version = "0.9.0";
 
 sub main::HELP_MESSAGE()
 {
@@ -25,13 +25,17 @@ sub main::HELP_MESSAGE()
     print("\t-p SERVER_PORT\tYour server port\n");
     print("\t-k PASSWORD\tYour top-secret password \n");
     print("\t-m METHOD\tYour favarite encryption method, pick one of the methods below:\n");
-    print("\taes-128-cbc aes-128-cfb aes-128-ctr aes-128-ofb\n");
-    print("\taes-192-cbc aes-192-cfb aes-192-ctr aes-192-ofb\n");
-    print("\taes-256-cbc aes-256-cfb aes-256-ctr aes-256-ofb\n");
-    print("\tcamellia-128-cbc camellia-128-cfb camellia-128-ctr camellia-128-ofb\n");
-    print("\tcamellia-192-cbc camellia-192-cfb camellia-192-ctr camellia-192-ofb\n");
-    print("\tcamellia-256-cbc camellia-256-cfb camellia-256-ctr camellia-256-ofb\n");
-    print("\tchacha20-ietf rc4-md5 rc4-sha rc6\n");
+    print("\taes-128-cfb aes-128-ctr aes-128-gcm aes-128-ofb\n");
+    print("\taes-192-cfb aes-192-ctr aes-192-gcm aes-192-ofb\n");
+    print("\taes-256-cfb aes-256-ctr aes-256-gcm aes-256-ofb\n");
+    print("\tcamellia-128-cfb camellia-128-ctr camellia-128-ofb\n");
+    print("\tcamellia-192-cfb camellia-192-ctr camellia-192-ofb\n");
+    print("\tcamellia-256-cfb camellia-256-ctr camellia-256-ofb\n");
+    print("\tchacha20-ietf chacha20-ietf-poly1305\n");
+    print("\trc4-md5 rc4-sha\n");
+    print("\trc6-128-cfb rc6-128-ctr rc6-128-ofb\n");
+    print("\trc6-192-cfb rc6-192-ctr rc6-192-ofb\n");
+    print("\trc6-256-cfb rc6-256-ctr rc6-256-ofb\n");
     printf("\t-c CONFIG\tName of your config.json file with full path\n");
     print("\t-h, --help\tDisplay help information\n");
     print("\t-v, --version\tDisplay version information\n");
@@ -99,12 +103,10 @@ ssserver.pl
 
 =head1 VERSION
 
-Version 0.8.1
+Version 0.9.0
 
 =head1 SYNOPSIS
 
-Shadowsocks protocol server.
-
 Usage: ssserver.pl -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
     -l LOCAL_PORT -k PASSWORD -m METHOD [-c CONFIG]
 
@@ -114,13 +116,17 @@ Usage: ssserver.pl -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
 	-p SERVER_PORT	Your server port
 	-k PASSWORD	Your top-secret password 
 	-m METHOD	Your favarite encryption method, pick one of the methods below:
-	aes-128-cbc aes-128-cfb aes-128-ctr aes-128-ofb
-	aes-192-cbc aes-192-cfb aes-192-ctr aes-192-ofb
-	aes-256-cbc aes-256-cfb aes-256-ctr aes-256-ofb
-	camellia-128-cbc camellia-128-cfb camellia-128-ctr camellia-128-ofb
-	camellia-192-cbc camellia-192-cfb camellia-192-ctr camellia-192-ofb
-	camellia-256-cbc camellia-256-cfb camellia-256-ctr camellia-256-ofb
-	chacha20-ietf rc4-md5 rc4-sha rc6
+	aes-128-cfb aes-128-ctr aes-128-gcm aes-128-ofb
+	aes-192-cfb aes-192-ctr aes-192-gcm aes-192-ofb
+	aes-256-cfb aes-256-ctr aes-256-gcm aes-256-ofb
+	camellia-128-cfb camellia-128-ctr camellia-128-ofb
+	camellia-192-cfb camellia-192-ctr camellia-192-ofb
+	camellia-256-cfb camellia-256-ctr camellia-256-ofb
+	chacha20-ietf chacha20-ietf-poly1305
+	rc4-md5 rc4-sha
+	rc6-128-cfb rc6-128-ctr rc6-128-ofb
+	rc6-192-cfb rc6-192-ctr rc6-192-ofb
+	rc6-256-cfb rc6-256-ctr rc6-256-ofb
 	-c CONFIG	Name of your config.json file with full path
 	-h, --help	Display help information
 	-v, --version	Display version information
@@ -128,6 +134,13 @@ Usage: ssserver.pl -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
 
 =head1 DESCRIPTION
 
+Shadowsocks protocol server.
+
+=head1 SEE ALSO
+
+L<Shadowsocks Official website |https://shadowsocks.org/en/index.html>,L<Shadowsocks on Wikipedia |https://en.wikipedia.org/wiki/Shadowsocks>
+
+
 Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭)
 protocol client and server , Shadowsocks is a secure transport protocol based on
 SOCKS Protocol Version 5 (RFC 1928 ).
