@@ -7,10 +7,11 @@ use strict;
 use warnings;
 use AnyEvent::Log;
 use Carp;
+use Crypt::AuthEnc::GCM;
 use Crypt::Cipher::AES;
 use Crypt::Cipher::Camellia;
 use Crypt::Cipher::RC6;
-use Crypt::Mode::CBC;
+use Crypt::KeyDerivation qw(hkdf);
 use Crypt::Mode::CFB;
 use Crypt::Mode::CTR;
 use Crypt::Mode::OFB;
@@ -18,42 +19,56 @@ use Crypt::NaCl::Sodium;
 use Crypt::Random qw( makerandom_octet );
 use Digest::MD5;
 use Digest::SHA;
-use Mcrypt qw(ARCFOUR RIJNDAEL_128 RIJNDAEL_192 RIJNDAEL_256);
-use Mcrypt qw(:MODES);
 use Mcrypt qw(:FUNCS);
+#use String::HexConvert ':all';
 
-our $VERSION = '0.8.2';
+our $VERSION = '0.9.0';
 
 our %_ciphers = 
     (
-        "rc4-md5"  => [Mcrypt::ARCFOUR,Mcrypt::STREAM,16,16],
-        "rc4-sha" => [Mcrypt::ARCFOUR,Mcrypt::STREAM,16,20],
-        "aes-128-cbc"  => [Mcrypt::RIJNDAEL_128,Mcrypt::CBC,16,16],
         "aes-128-cfb"  => ['AES','cfb',16,16],
+        #"aes-128-cfb"  => ['rijndael-128','ncfb',16,16],
         "aes-128-ctr"  => ['AES','ctr',16,16],
-        "aes-128-ofb" => [Mcrypt::RIJNDAEL_128,Mcrypt::OFB,16,16],
-        "aes-192-cbc"  => [Mcrypt::RIJNDAEL_192,Mcrypt::CBC,24,16],     
+        # "aes-128-ctr"  => ['rijndael-128','ctr',16,16],
+         "aes-128-gcm"  => ['AES',undef,16,16],
+        #"aes-128-ofb" => ['rijndael-128','nofb',16,16],
+        "aes-128-ofb" => ['AES','ofb',16,16],    
         "aes-192-cfb" => ['AES','cfb',24,16],
+        #"aes-192-cfb"  => ['rijndael-128','ncfb',24,16],
         "aes-192-ctr"  => ['AES','ctr',24,16],
-        "aes-192-ofb" => [Mcrypt::RIJNDAEL_192,Mcrypt::OFB,24,16],
-        "aes-256-cbc"  => [Mcrypt::RIJNDAEL_256,Mcrypt::CBC,32,16],
+        #"aes-192-ctr"  => ['rijndael-128','ctr',24,16],
+         "aes-192-gcm"  => ['AES',undef,24,16],
+        #"aes-192-ofb" => ['rijndael-128','nofb',24,16],
+        "aes-192-ofb" => ['AES','ofb',24,16],
         "aes-256-cfb"  => ['AES','cfb',32,16],
+        #"aes-256-cfb"  => ['rijndael-128','ncfb',32,16],
         "aes-256-ctr"  => ['AES','ctr',32,16],
-        "aes-256-ofb"  => [Mcrypt::RIJNDAEL_256,Mcrypt::OFB,32,16],
-        "camellia-128-cbc" => ['Camellia','cbc',16,16],
+        #"aes-256-ctr"  => ['rijndael-128','ctr',32,16],
+        "aes-256-gcm"  => ['AES',undef,32,16],
+        #"aes-256-ofb"  => ['rijndael-128','nofb',32,16],
+        "aes-256-ofb"  => ['AES','ofb',32,16],
         "camellia-128-cfb" => ['Camellia','cfb',16,16],
         "camellia-128-ctr" => ['Camellia','ctr',16,16],
         "camellia-128-ofb" => ['Camellia','ofb',16,16],
-        "camellia-192-cbc" => ['Camellia','cbc',24,16],
         "camellia-192-cfb" => ['Camellia','cfb',24,16],
         "camellia-192-ctr" => ['Camellia','ctr',24,16],
         "camellia-192-ofb" => ['Camellia','ofb',24,16],
-        "camellia-256-cbc" => ['Camellia','cbc',32,16],
         "camellia-256-cfb" => ['Camellia','cfb',32,16],
         "camellia-256-ctr" => ['Camellia','ctr',32,16],
         "camellia-256-ofb" => ['Camellia','ofb',32,16],
         "chacha20-ietf" => [undef,undef,32,12],
-        "rc6" => [undef,undef,8,16]
+        "chacha20-ietf-poly1305" =>  [undef,undef,32,12],
+         "rc4-md5"  => ['arcfour','stream',16,16],
+        "rc4-sha" => ['arcfour','stream',16,20],
+        "rc6-128-cfb" => ['RC6','cfb',16,16],
+        "rc6-128-ctr" => ['RC6','ctr',16,16],
+        "rc6-128-ofb" => ['RC6','ofb',16,16],
+        "rc6-192-cfb" => ['RC6','cfb',24,16],
+        "rc6-192-ctr" => ['RC6','ctr',24,16],
+        "rc6-192-ofb" => ['RC6','ofb',24,16],
+        "rc6-256-cfb" => ['RC6','cfb',32,16],
+        "rc6-256-ctr" => ['RC6','ctr',32,16],
+        "rc6-256-ofb" => ['RC6','ofb',32,16],
     );
 
 sub _EVP_BytesToKey($$$$$) 
@@ -104,13 +119,15 @@ sub _initialize_cipher($$)
     my $_encryptor;
     my $_decryptor;
     my $_key;
+    my $_encrypt_subkey;
     my $_iv;
-    my $_nonce;
+    my $_encrypt_nonce;
+    my $_decrypt_nonce;
     if (!defined($_method))
     {
         AE::log info => "Encryption method undefinde, using RC4-MD5." ;
-        $_encryptor = Mcrypt::mcrypt_load(Mcrypt::ARCFOUR,'',Mcrypt::STREAM , '' );
-        $_decryptor = Mcrypt::mcrypt_load(Mcrypt::ARCFOUR ,'',Mcrypt::STREAM, '' );
+        $_encryptor = Mcrypt::mcrypt_load('arcfour','','stream' , '' );
+        $_decryptor = Mcrypt::mcrypt_load('arcfour' ,'','stream', '' );
         $_key = _EVP_BytesToKey(16, 16, '', $_password, 1 );
         $_iv = makerandom_octet( Length => 16 );
         my $md = Digest::MD5->new();
@@ -120,101 +137,184 @@ sub _initialize_cipher($$)
         }
         else
         {
-            if($_method eq 'chacha20-ietf')
+            if($_method =~ /^chacha20/)
             {
-                $_encryptor = Crypt::NaCl::Sodium->stream();
-                $_key = $_encryptor->chacha20_keygen();
-                $_nonce = $_encryptor->chacha20_ietf_nonce();
-                $_decryptor = Crypt::NaCl::Sodium->stream();
-                $_iv = $_nonce;
+                 if ($_method eq 'chacha20-ietf')
+                 {
+                    $_encryptor = Crypt::NaCl::Sodium->stream();
+                    #$_key = $_encryptor->chacha20_keygen();
+                    $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
+                    $_encrypt_nonce = $_encryptor->chacha20_ietf_nonce();
+                    $_decryptor = Crypt::NaCl::Sodium->stream();
+                    $_iv = $_encrypt_nonce;
+                }
+                else
+                {
+                    $_encryptor = Crypt::NaCl::Sodium->aead();
+                    #$_key = $_encryptor->chacha20_keygen();
+                    $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
+                    my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
+                    $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
+                    #carp ascii_to_hex($_encrypt_subkey);
+                    $_decryptor = Crypt::NaCl::Sodium->aead();
+                    $_iv = $_encrypt_salt;
+                    $_encrypt_nonce = $_encryptor->ietf_nonce("\0");
+                    $_decrypt_nonce = $_decryptor->ietf_nonce("\0");
+                    #carp $_encrypt_nonce;
+                    #carp ascii_to_hex($_encrypt_nonce);
                 }
-            elsif ( $_method eq 'rc6') 
-            {
-                $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
-                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
-                $_encryptor = Crypt::Mode::CBC->new('RC6');
-                $_decryptor = Crypt::Mode::CBC->new('RC6');
             }
-            elsif(defined($_method) and $_method =~ /cfb$/)
+             elsif($_method =~ /gcm$/ )
             {
                 $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
-                $_encryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
-                $_decryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
+                my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
+                my $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
+                $_iv = $_encrypt_salt; 
+                $_encryptor = Crypt::AuthEnc::GCM->new('AES', $_encrypt_subkey);
+                $_encrypt_nonce = Data::BytesLocker->new("\0" x 12);
+                $_decrypt_nonce = Data::BytesLocker->new("\0" x 12);
             }
-             elsif(defined($_method) and $_method =~ /ctr$/)
-            {
-                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
-                $_encryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
-                $_decryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
-            }
-             elsif(defined($_method) and $_method =~ /^camellia/)
-            {
-                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
-                if ($_method =~ /cbc$/)
-                { 
-                $_encryptor = Crypt::Mode::CBC->new($_ciphers{$_method}->[0]);
-                $_decryptor = Crypt::Mode::CBC->new($_ciphers{$_method}->[0]);
-                }
-                elsif($_method =~ /cfb$/)
-                {
-                $_encryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
-                $_decryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
-                }
-                elsif($_method =~ /ctr$/)
-                {
-                $_encryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
-                $_decryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
-                }
-                elsif($_method =~ /ofb$/)
-                {
-                $_encryptor = Crypt::Mode::OFB->new($_ciphers{$_method}->[0]);
-                $_decryptor = Crypt::Mode::OFB->new($_ciphers{$_method}->[0]);
-                }
-            }
-            elsif(defined($_method) and $_method !~ /cfb$/ and $_method !~ /ctr$/  and $_method !~ /^camellia/)
+           
+            elsif($_method =~ /^rc4/ )
             {
                 $_encryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0],'',$_ciphers{$_method}->[1] , '' );
                 $_decryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0] ,'',$_ciphers{$_method}->[1], '' );
                 $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
                 $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
-                if ($_method eq 'rc4-md5' )
-                {
-                   my $md = Digest::MD5->new();
-                   $md->add($_key . $_iv);
-                   my $encrypt_rc4_key = $md->digest();
-                   Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );
-                }
-                elsif($_method eq 'rc4-sha' )
-                {
-                   my $sha = Digest::SHA->new();
-                   $sha->add($_key . $_iv);
-                   my $encrypt_rc4_key = substr($sha->digest(),0,16);
-                   Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );
-                }
-                elsif($_method ne 'rc6')
-                {
-                   Mcrypt::mcrypt_init($_encryptor, $_key, $_iv );
-                }
-           }
+
+                    my $encrypt_rc4_key;
+                    if ($_method eq 'rc4-md5' )
+                    {
+                       my $md = Digest::MD5->new();
+                       $md->add($_key . $_iv);
+                       $encrypt_rc4_key = $md->digest();
+                    }
+                    elsif($_method eq 'rc4-sha' )
+                    {
+                       my $sha = Digest::SHA->new();
+                       $sha->add($_key . $_iv);
+                       $encrypt_rc4_key = substr($sha->digest(),0,16);
+                    }
+                    Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );
+            }
+             
+            elsif( ($_method =~ /^aes/  or $_method =~ /^camellia/ or $_method =~ /^rc6/ ) and $_method !~ /gcm$/ )
+            {
+                          if($_method =~ /cfb$/)
+            {
+                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+                $_encryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
+            }
+             elsif($_method =~ /ctr$/)
+            {
+                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+                $_encryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
+            }
+            elsif($_method =~ /ofb$/)
+            {
+                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+                $_encryptor = Crypt::Mode::OFB->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::OFB->new($_ciphers{$_method}->[0]);
+            }
+        }
     }
-    return $_encryptor, $_decryptor, $_key, $_iv, $_nonce;
+    return $_encryptor, $_decryptor, $_key, $_encrypt_subkey,$_iv, $_encrypt_nonce,$_decrypt_nonce;
 }
 
 sub _get_algorithm($)
 {
     my $_method   = shift;
-    return $_ciphers{$_method}->[0];
+    if (defined($_method))
+    {
+        return $_ciphers{$_method}->[0];
+    }
+    else
+    {
+        return 'arcfour';
+    }
     }
 
 sub _get_mode($)
 {
     my $_method   = shift;
-    return $_ciphers{$_method}->[1];
+    if (defined($_method))
+    {
+        return $_ciphers{$_method}->[1];
     }
+    else
+    {
+        return 'stream'; 
+    }
+}
 
+sub _get_key_size($)
+{
+    my $_method = shift;
+    if (defined($_method))
+    {
+        return $_ciphers{$_method}->[2];   
+    }
+    else
+    {
+        return 16;
+    }
+}
+
+sub _get_iv_size($)
+{
+    my $_method = shift;
+    if (defined($_method))
+    {
+        return $_ciphers{$_method}->[3];   
+    }
+    else
+    {
+        return 16;
+    }
+}
+
+sub _add_padding($$$)
+{
+        # add required padding so we can recover the original string length after decryption
+        # (padding bytes have value set to padding length)
+        my $_method = shift;
+        my $_pt = shift;
+        my $_pad_len = shift;
+        my $_block_size = _get_key_size($_method);
+        #my $_pt_len = length($_pt)  % $_block_size;
+        #my $_pad_len = $_block_size - $_pt_len;
+        $_pt .= (chr($_pad_len)) x $_pad_len unless $_pad_len == $_block_size;
+        #$_pt = (chr(0)) x $_pad_len . $_pt unless $_pad_len == $_block_size;
+        return $_pt;
+}
+
+
+
+sub _remove_padding($$$)
+{
+        # remove padding if necessary (padding byte value gives length of padding)
+        my $_method = shift;
+        my $_ct = shift;
+        my $_pad_len = shift;
+        my $_ct_len = length($_ct);
+        my $_block_size = _get_key_size($_method);
+        #my $_pad_len = ord(substr($_ct, -1, 1));
+        if ($_pad_len > $_block_size)
+        {
+            AE::log error => "invalid pad byte";
+        }
+        else
+        {
+        $_ct  = substr($_ct, 0, $_ct_len - $_pad_len);
+        #$_ct  = substr($_ct, $_pad_len, $_ct_len - $_pad_len);
+        }
+        return $_ct;
+}
 1;    # End of Net::Shadowsocks
 
  __END__
@@ -225,49 +325,50 @@ sub _get_mode($)
 
 =head1 NAME
 
-Net::Shadowsocks
+Net::Shadowsocks - the asynchronous, non-blocking shadowsocks client and server. 
 
 =head1 VERSION
 
-Version 0.8.2
+Version 0.9.0
 
 =head1 SYNOPSIS
-
-    ssclient.pl is the asynchronous, non-blocking shadowsocks client. 
-    ssserver.pl is the asynchronous, non-blocking shadowsocks server. 
-    Run ssclient.pl and/or ssserver.pl and follow instructions.
     
 =head1 DESCRIPTION
 
-Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭)
-protocol client and server , Shadowsocks is a secure transport protocol based on
-SOCKS Protocol Version 5 (RFC 1928 ). 
+Shadowsocks is a secure transport protocol based on SOCKS Protocol Version 5 (RFC 1928 ).Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭) protocol client and server. ssclient.pl is the asynchronous, non-blocking shadowsocks client. ssserver.pl is the asynchronous, non-blocking shadowsocks server. Run ssclient.pl and/or ssserver.pl and follow instructions.
 
-1. A total of 28 encryption methods are supported:
+1. A total of 34 encryption methods are supported:
 
-AES-128-CBC AES-128-CFB AES-128-CTR AES-128-OFB 
-AES-192-CBC AES-192-CFB AES-192-CTR AES-192-OFB 
-AES-256-CBC AES-256-CFB AES-256-CTR AES-256-OFB
-Camellia-128-CBC Camellia-128-CFB Camellia-128-CTR Camellia-128-OFB 
-Camellia-192-CBC Camellia-192-CFB Camellia-192-CTR Camellia-192-OFB 
-Camellia-256-CBC Camellia-256-CFB Camellia-256-CTR Camellia-256-OFB
-Chacha20-IETF
-RC4-MD5 RC4-SHA RC6 
+	aes-128-cfb aes-128-ctr aes-128-gcm aes-128-ofb
+	aes-192-cfb aes-192-ctr aes-192-gcm aes-192-ofb
+	aes-256-cfb aes-256-ctr aes-256-gcm aes-256-ofb
+	camellia-128-cfb camellia-128-ctr camellia-128-ofb
+	camellia-192-cfb camellia-192-ctr camellia-192-ofb
+	camellia-256-cfb camellia-256-ctr camellia-256-ofb
+	chacha20-ietf chacha20-ietf-poly1305
+	rc4-md5 rc4-sha
+	rc6-128-cfb rc6-128-ctr rc6-128-ofb
+	rc6-192-cfb rc6-192-ctr rc6-192-ofb
+	rc6-256-cfb rc6-256-ctr rc6-256-ofb
 
 2.The following ciphers deprecated by Shadowsocks are not supported: 
-bf-cfb chacha20 salsa20
+
+      bf-cfb chacha20 salsa20
 
 3.The following ciphers recommended by Shadowsocks are not supported yet: 
-aes-128-gcm aes-192-gcm aes-256-gcm 
-chacha20-ietf-poly1305 
-xchacha20-ietf-poly1305 
+ 
+      xchacha20-ietf-poly1305 
 
 Please note TLS 1.2 has removed IDEA and DES cipher suites. and because of 
 CVE-2016-2183,  http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2016-2183
-, this module has removed all support for DES and 3DES ciphers.
+, this module has removed all support for DES and 3DES ciphers. 
 
 Project website https://osdn.net/projects/ssperl/
 
+=head1 SEE ALSO
+
+L<Shadowsocks Official website |https://shadowsocks.org/en/index.html>,L<Shadowsocks on Wikipedia |https://en.wikipedia.org/wiki/Shadowsocks>
+
 =head1 AUTHOR
 
 Li ZHOU, C<< <lzh at cpan.org> >>
